@@ -70,9 +70,15 @@ def fetch_user_details(access_token, environment: ENVIRONMENTS) -> dict:
         result = r.json()
         logging.debug(f"{result=}")
     except httpx.HTTPStatusError as e:
-        if e.response.status_code == 401:
+        if e.response.status_code == 400 and "API access" in e.response.json().get(
+            "message"
+        ):
             raise misc.AuthenticationError(
-                f"Authentication failed, check access token and environment"
+                f"Authentication failed. User lacks API permissions"
+            ) from None
+        elif e.response.status_code == 401:
+            raise misc.AuthenticationError(
+                f"Authentication failed. Check access token and environment"
             ) from None
         else:
             raise e from None
@@ -172,9 +178,13 @@ async def fetch_status_single_async(client, guid, url, headers):
         if status not in GOOD_STATUSES:
             with logging_redirect_tqdm():
                 logging.info(f"{guid} has status {status}")
+    elif r.status_code == 400 and "API access" in r.json().get("message"):
+        raise misc.AuthenticationError(
+            f"Authentication failed (HTTP {r.status_code}). User lacks API permissions"
+        )
     elif r.status_code == 401:
-        raise RuntimeError(
-            f"Authorisation failed (HTTP {r.status_code}). Invalid token?"
+        raise misc.AuthenticationError(
+            f"Authentication failed (HTTP {r.status_code}). Invalid token?"
         )
     elif r.json().get("message") == "Sample not found.":
         status = "UNKNOWN"
@@ -273,6 +283,14 @@ async def download_single_async(
             update_fasta_header(
                 Path(f"{prefix}.{file_types_extensions[file_type]}"), guid, name
             )
+    elif r.status_code == 400 and "API access" in r.json().get("message"):
+        raise misc.AuthenticationError(
+            f"Bad request (HTTP {r.status_code}). User lacks API permissions"
+        )
+    elif r.status_code == 401:
+        raise misc.AuthenticationError(
+            f"Authorisation failed (HTTP {r.status_code}). Invalid token?"
+        )
     else:
         r.raise_for_status()  # Raises retryable httpx.HTTPError
 
@@ -476,6 +494,7 @@ class Batch:
         working_dir: Path = Path("/tmp"),
         out_dir: Path = Path(),
         processes: int = 0,
+        connections: int = 10,
         environment: ENVIRONMENTS = DEFAULT_ENVIRONMENT,
         json_messages: bool = False,
         save_reads: bool = False,
@@ -489,6 +508,7 @@ class Batch:
         self.out_dir = out_dir
         out_dir.mkdir(parents=False, exist_ok=True)
         self.processes = processes if processes else int(multiprocessing.cpu_count())
+        self.connections = connections
         self.json = {"validation": "", "decontamination": "", "submission": ""}
         self.json_messages = json_messages
         self.samtools_path = misc.get_binary_path("samtools")
@@ -711,6 +731,7 @@ class Batch:
         target_path = self.out_dir / Path(self.batch_guid + ".mapping.csv")
         combined_df.to_csv(target_path, index=False)
         self.mapping_path = target_path
+        logging.info(f"Saved mapping CSV to {self.mapping_path}")
 
     def _fetch_par(self):
         """Private method that calls ORDS to get a Pre-Authenticated Request.
@@ -766,18 +787,27 @@ class Batch:
         if self.json_messages:
             misc.print_progress_message_json(action="upload", status="started")
         uploads = self._get_uploads()
-        process_map(
-            partial(
-                misc.upload_sample,
-                headers=self.headers,
-                json_messages=self.json_messages,
-            ),
-            uploads,
-            max_workers=10,
-            desc=f"Uploading {len(uploads)} sample(s) (10 connections)",
-            bar_format="{l_bar}{bar}| {n_fmt}/{total_fmt}",
-            leave=False,
-        )
+        if self.connections == 1:
+            logging.debug("Single upload process")
+            for upload in uploads:
+                misc.upload_sample(
+                    upload=upload,
+                    headers=self.headers,
+                    json_messages=self.json_messages,
+                )
+        else:
+            process_map(
+                partial(
+                    misc.upload_sample,
+                    headers=self.headers,
+                    json_messages=self.json_messages,
+                ),
+                uploads,
+                max_workers=self.connections,
+                desc=f"Uploading {len(uploads)} sample(s)",
+                bar_format="{l_bar}{bar}| {n_fmt}/{total_fmt}",
+                leave=False,
+            )
         self.uploaded = True
         if self.json_messages:
             misc.print_progress_message_json(action="upload", status="finished")
@@ -799,7 +829,7 @@ class Batch:
                 url=endpoint,
                 data=json.dumps(submission, ensure_ascii=False).encode("utf-8"),
                 headers=headers,
-                timeout=60,
+                timeout=80,
             )
             logging.debug(f"post_submission(): {r.text=}")
             r.raise_for_status()
@@ -826,9 +856,7 @@ class Batch:
         self.submission["batch"]["uploader"]["upload_finish"] = misc.oracle_timestamp()
         post_submission(self.submission, self.headers)
         put_done_mark(self.batch_guid, self.headers)
-        logging.info(
-            f"Submitted batch {self.batch_guid}, mapping CSV saved to {self.mapping_path}"
-        )
+        logging.info(f"Submitted batch {self.batch_guid}")
         success_message = {
             "submission": {
                 "status": "success",
@@ -936,8 +964,9 @@ class Batch:
 
     def upload(self, dry_run: bool = False, save_reads: bool = False) -> None:
         try:
-            if self.processes > 1:
-                logging.info(f"Using {self.processes} processes")
+            logging.info(
+                f"Using {self.processes} process(es), {self.connections} connection(s)"
+            )
             self._decontaminate()
             if not self.headers:
                 logging.warning("No token provided, quitting")
@@ -957,10 +986,13 @@ class Batch:
     def _number_runs(self) -> None:
         """Enumerate unique values of run_number for submission"""
         samples_runs = self._get_sample_attrs("run_number")
+        logging.debug(f"{samples_runs=}")
         samples_run_numbers = misc.number_runs(samples_runs)
+        logging.debug(f"{samples_run_numbers=}")
         for s in self.samples:
             s.gpas_run_number = samples_run_numbers[s.sample_name]
         self.run_numbers = list(filter(None, samples_run_numbers.values()))
+        logging.debug(f"{self.run_numbers=}")
 
 
 def parse_decontamination_stats(stdout: str) -> dict:
